@@ -19,6 +19,7 @@ are what gets used.
 """
 
 import re, time
+from array import array
 try:
     import cPickle as pickle
 except ImportError:
@@ -30,18 +31,12 @@ import constraints as consts
 from formencode import compound
 from formencode import validators
 from classregistry import findClass
-from converters import array_type, buffer_type
-from util.backports import count
+from itertools import count
 
 NoDefault = sqlbuilder.NoDefault
-True, False = 1==1, 0==1
 
-try:
-    import datetime
-except ImportError: # Python 2.2
-    datetime_available = False
-else:
-    datetime_available = True
+import datetime
+datetime_available = True
 
 try:
     from mx import DateTime
@@ -66,18 +61,11 @@ if mxdatetime_available:
         DateTimeType = type(DateTime.DateTime())
         TimeType = type(DateTime.DateTime.Time(DateTime.DateTime()))
 
-if datetime_available:
-    default_datetime_implementation = DATETIME_IMPLEMENTATION
-elif mxdatetime_available:
-    default_datetime_implementation = MXDATETIME_IMPLEMENTATION
-else:
-    default_datetime_implementation = None
+default_datetime_implementation = DATETIME_IMPLEMENTATION
 
 __all__ = ["datetime_available", "mxdatetime_available",
-    "default_datetime_implementation"
-]
-if datetime_available:
-    __all__.append("DATETIME_IMPLEMENTATION")
+        "default_datetime_implementation", "DATETIME_IMPLEMENTATION"]
+
 if mxdatetime_available:
     __all__.append("MXDATETIME_IMPLEMENTATION")
 
@@ -161,7 +149,7 @@ class SOCol(object):
                 "The only string value allowed for cascade is 'null' (you gave: %r)" % cascade)
         self.cascade = cascade
 
-        if type(constraints) not in (type([]), type(())):
+        if not isinstance(constraints, (list, tuple)):
             constraints = [constraints]
         self.constraints = self.autoConstraints() + constraints
 
@@ -433,9 +421,9 @@ class Col(object):
 class SOStringLikeCol(SOCol):
     """A common ancestor for SOStringCol and SOUnicodeCol"""
     def __init__(self, **kw):
-        self.length = popKey(kw, 'length')
-        self.varchar = popKey(kw, 'varchar', 'auto')
-        self.char_binary = popKey(kw, 'char_binary', None) # A hack for MySQL
+        self.length = kw.pop('length', None)
+        self.varchar = kw.pop('varchar', 'auto')
+        self.char_binary = kw.pop('char_binary', None) # A hack for MySQL
         if not self.length:
             assert self.varchar == 'auto' or not self.varchar, \
                    "Without a length strings are treated as TEXT, not varchar"
@@ -477,7 +465,7 @@ class SOStringLikeCol(SOCol):
 
     def _sqliteType(self):
         self._check_case_sensitive("SQLite")
-        return super(SOStringLikeCol, self)._sqliteType()
+        return super(SOStringLikeCol, self)._sqliteType().replace("CHAR(", "CHAR (")
 
     def _sybaseType(self):
         self._check_case_sensitive("SYBASE")
@@ -550,7 +538,7 @@ class UnicodeStringValidator(validators.Validator):
             return None
         if isinstance(value, unicode):
             return value
-        if isinstance(value, array_type): # MySQL
+        if isinstance(value, array): # MySQL
             return unicode(value.tostring(), self.db_encoding)
         return unicode(value, self.db_encoding)
 
@@ -563,7 +551,7 @@ class UnicodeStringValidator(validators.Validator):
 
 class SOUnicodeCol(SOStringLikeCol):
     def __init__(self, **kw):
-        self.dbEncoding = popKey(kw, 'dbEncoding', 'UTF-8')
+        self.dbEncoding = kw.pop('dbEncoding', 'UTF-8')
         super(SOUnicodeCol, self).__init__(**kw)
 
     def createValidators(self):
@@ -582,10 +570,7 @@ class IntValidator(validators.Validator):
         if isinstance(value, (int, long, sqlbuilder.SQLExpression)):
             return value
         try:
-            try:
-                return int(value)
-            except OverflowError: # for Python 2.2
-                return long(value)
+            return int(value)
         except:
             raise validators.Invalid("expected an int in the IntCol '%s', got %s %r instead" % \
                 (self.name, type(value), value), value, state)
@@ -601,9 +586,9 @@ class IntValidator(validators.Validator):
 class SOIntCol(SOCol):
     # 3-03 @@: support precision, maybe max and min directly
     def __init__(self, **kw):
-        self.length = popKey(kw, 'length')
-        self.unsigned = bool(popKey(kw, 'unsigned'))
-        self.zerofill = bool(popKey(kw, 'zerofill'))
+        self.length = kw.pop('length', None)
+        self.unsigned = bool(kw.pop('unsigned', None))
+        self.zerofill = bool(kw.pop('zerofill', None))
         SOCol.__init__(self, **kw)
 
     def autoConstraints(self):
@@ -667,17 +652,17 @@ class BoolValidator(validators.Validator):
         if value is None:
             return None
         elif not value:
-            return sqlbuilder.FALSE
+            return False
         else:
-            return sqlbuilder.TRUE
+            return True
 
     def from_python(self, value, state):
         if value is None:
             return None
         elif value:
-            return sqlbuilder.TRUE
+            return True
         else:
-            return sqlbuilder.FALSE
+            return False
 
 class SOBoolCol(SOCol):
     def autoConstraints(self):
@@ -706,7 +691,7 @@ class SOBoolCol(SOCol):
         return "BOOLEAN"
 
     def _sqliteType(self):
-        return "TINYINT"
+        return "BOOLEAN"
 
 class BoolCol(Col):
     baseClass = SOBoolCol
@@ -788,7 +773,7 @@ class SOForeignKey(SOKeyCol):
 
     def sqliteCreateSQL(self):
         sql = SOKeyCol.sqliteCreateSQL(self)
-        other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
+        other = findClass(self.foreignKey)
         tName = other.sqlmeta.table
         idName = other.sqlmeta.idName
         if self.cascade is not None:
@@ -870,7 +855,7 @@ class SOForeignKey(SOKeyCol):
 
     def sybaseCreateSQL(self):
         sql = SOKeyCol.sybaseCreateSQL(self)
-        other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
+        other = findClass(self.foreignKey)
         tName = other.sqlmeta.table
         idName = other.sqlmeta.idName
         reference = ('REFERENCES %(tName)s(%(idName)s) ' %
@@ -885,7 +870,7 @@ class SOForeignKey(SOKeyCol):
 
     def mssqlCreateSQL(self):
         sql = SOKeyCol.mssqlCreateSQL(self)
-        other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
+        other = findClass(self.foreignKey)
         tName = other.sqlmeta.table
         idName = other.sqlmeta.idName
         reference = ('REFERENCES %(tName)s(%(idName)s) ' %
@@ -899,7 +884,7 @@ class SOForeignKey(SOKeyCol):
         return None
 
     def maxdbCreateSQL(self):
-        other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
+        other = findClass(self.foreignKey)
         fidName = self.dbName
         #I assume that foreign key name is identical to the id of the reference table
         sql = ' '.join([fidName, self._maxdbType()])
@@ -923,7 +908,7 @@ class ForeignKey(KeyCol):
 class SOEnumCol(SOCol):
 
     def __init__(self, **kw):
-        self.enumValues = popKey(kw, 'enumValues', None)
+        self.enumValues = kw.pop('enumValues', None)
         assert self.enumValues is not None, \
                'You must provide an enumValues keyword argument'
         super(SOEnumCol, self).__init__(**kw)
@@ -951,7 +936,7 @@ class SOEnumCol(SOCol):
         return "VARCHAR(%i) %s" % (length, checkConstraint)
 
     def _sqliteType(self):
-        return self._postgresType()
+        return self._postgresType().replace("CHAR(", "CHAR (")
 
     def _sybaseType(self):
         return self._postgresType()
@@ -1014,7 +999,7 @@ class SetValidator(validators.Validator):
 
 class SOSetCol(SOCol):
     def __init__(self, **kw):
-        self.setValues = popKey(kw, 'setValues', None)
+        self.setValues = kw.pop('setValues', None)
         assert self.setValues is not None, \
                 'You must provide a setValues keyword argument'
         super(SOSetCol, self).__init__(**kw)
@@ -1033,43 +1018,42 @@ class SetCol(Col):
     baseClass = SOSetCol
 
 
-if datetime_available:
-    class DateTimeValidator(validators.DateValidator):
-        def to_python(self, value, state):
-            if value is None:
-                return None
-            if isinstance(value, (datetime.datetime, datetime.date, datetime.time, sqlbuilder.SQLExpression)):
-                return value
-            if mxdatetime_available:
-                if isinstance(value, DateTimeType):
-                    # convert mxDateTime instance to datetime
-                    if (self.format.find("%H") >= 0) or (self.format.find("%T")) >= 0:
-                        return datetime.datetime(value.year, value.month, value.day,
-                            value.hour, value.minute, int(value.second))
-                    else:
-                        return datetime.date(value.year, value.month, value.day)
-                elif isinstance(value, TimeType):
-                    # convert mxTime instance to time
-                    if self.format.find("%d") >= 0:
-                        return datetime.timedelta(seconds=value.seconds)
-                    else:
-                        return datetime.time(value.hour, value.minute, int(value.second))
-            try:
-                stime = time.strptime(value, self.format)
-            except:
-                raise validators.Invalid("expected a date/time string of the '%s' format in the DateTimeCol '%s', got %s %r instead" % \
-                    (self.format, self.name, type(value), value), value, state)
-            return datetime.datetime(*stime[:6])
+class DateTimeValidator(validators.DateValidator):
+    def to_python(self, value, state):
+        if value is None:
+            return None
+        if isinstance(value, (datetime.datetime, datetime.date, datetime.time, sqlbuilder.SQLExpression)):
+            return value
+        if mxdatetime_available:
+            if isinstance(value, DateTimeType):
+                # convert mxDateTime instance to datetime
+                if (self.format.find("%H") >= 0) or (self.format.find("%T")) >= 0:
+                    return datetime.datetime(value.year, value.month, value.day,
+                        value.hour, value.minute, int(value.second))
+                else:
+                    return datetime.date(value.year, value.month, value.day)
+            elif isinstance(value, TimeType):
+                # convert mxTime instance to time
+                if self.format.find("%d") >= 0:
+                    return datetime.timedelta(seconds=value.seconds)
+                else:
+                    return datetime.time(value.hour, value.minute, int(value.second))
+        try:
+            stime = time.strptime(value, self.format)
+        except:
+            raise validators.Invalid("expected a date/time string of the '%s' format in the DateTimeCol '%s', got %s %r instead" % \
+                (self.format, self.name, type(value), value), value, state)
+        return datetime.datetime(*stime[:6])
 
-        def from_python(self, value, state):
-            if value is None:
-                return None
-            if isinstance(value, (datetime.datetime, datetime.date, datetime.time, sqlbuilder.SQLExpression)):
-                return value
-            if hasattr(value, "strftime"):
-                return value.strftime(self.format)
-            raise validators.Invalid("expected a datetime in the DateTimeCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
+    def from_python(self, value, state):
+        if value is None:
+            return None
+        if isinstance(value, (datetime.datetime, datetime.date, datetime.time, sqlbuilder.SQLExpression)):
+            return value
+        if hasattr(value, "strftime"):
+            return value.strftime(self.format)
+        raise validators.Invalid("expected a datetime in the DateTimeCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
 if mxdatetime_available:
     class MXDateTimeValidator(validators.DateValidator):
@@ -1078,14 +1062,13 @@ if mxdatetime_available:
                 return None
             if isinstance(value, (DateTimeType, TimeType, sqlbuilder.SQLExpression)):
                 return value
-            if datetime_available: # convert datetime instance to mxDateTime
-                if isinstance(value, datetime.datetime):
-                    return DateTime.DateTime(value.year, value.month, value.day,
-                        value.hour, value.minute, value.second)
-                elif isinstance(value, datetime.date):
-                    return DateTime.Date(value.year, value.month, value.day)
-                elif isinstance(value, datetime.time):
-                    return DateTime.Time(value.hour, value.minute, value.second)
+            if isinstance(value, datetime.datetime):
+                return DateTime.DateTime(value.year, value.month, value.day,
+                    value.hour, value.minute, value.second)
+            elif isinstance(value, datetime.date):
+                return DateTime.Date(value.year, value.month, value.day)
+            elif isinstance(value, datetime.time):
+                return DateTime.Time(value.hour, value.minute, value.second)
             try:
                 stime = time.strptime(value, self.format)
             except:
@@ -1107,7 +1090,7 @@ class SODateTimeCol(SOCol):
     datetimeFormat = '%Y-%m-%d %H:%M:%S'
 
     def __init__(self, **kw):
-        datetimeFormat = popKey(kw, 'datetimeFormat')
+        datetimeFormat = kw.pop('datetimeFormat', None)
         if datetimeFormat:
             self.datetimeFormat = datetimeFormat
         super(SODateTimeCol, self).__init__(**kw)
@@ -1156,25 +1139,25 @@ class DateTimeCol(Col):
                        % DATETIME_IMPLEMENTATION)
     now = staticmethod(now)
 
-if datetime_available:
-    class DateValidator(DateTimeValidator):
-        def to_python(self, value, state):
-            if isinstance(value, datetime.datetime):
-                value = value.date()
-            if isinstance(value, datetime.date):
-                return value
-            value = super(DateValidator, self).to_python(value, state)
-            if isinstance(value, datetime.datetime):
-                value = value.date()
-            return value
 
-        from_python = to_python
+class DateValidator(DateTimeValidator):
+    def to_python(self, value, state):
+        if isinstance(value, datetime.datetime):
+            value = value.date()
+        if isinstance(value, datetime.date):
+            return value
+        value = super(DateValidator, self).to_python(value, state)
+        if isinstance(value, datetime.datetime):
+            value = value.date()
+        return value
+
+    from_python = to_python
 
 class SODateCol(SOCol):
     dateFormat = '%Y-%m-%d'
 
     def __init__(self, **kw):
-        dateFormat = popKey(kw, 'dateFormat')
+        dateFormat = kw.pop('dateFormat', None)
         if dateFormat: self.dateFormat = dateFormat
         super(SODateCol, self).__init__(**kw)
 
@@ -1217,27 +1200,26 @@ class DateCol(Col):
     baseClass = SODateCol
 
 
-if datetime_available:
-    class TimeValidator(DateTimeValidator):
-        def to_python(self, value, state):
-            if isinstance(value, datetime.time):
-                return value
-            if isinstance(value, datetime.timedelta):
-                if value.days:
-                    raise validators.Invalid(
-                        "the value for the TimeCol '%s' must has days=0, it has days=%d" %
-                            (self.name, value.days), value, state)
-                return datetime.time(*time.gmtime(value.seconds)[3:6])
-            value = super(TimeValidator, self).to_python(value, state)
-            if isinstance(value, datetime.datetime):
-                value = value.time()
+class TimeValidator(DateTimeValidator):
+    def to_python(self, value, state):
+        if isinstance(value, datetime.time):
             return value
+        if isinstance(value, datetime.timedelta):
+            if value.days:
+                raise validators.Invalid(
+                    "the value for the TimeCol '%s' must has days=0, it has days=%d" %
+                        (self.name, value.days), value, state)
+            return datetime.time(*time.gmtime(value.seconds)[3:6])
+        value = super(TimeValidator, self).to_python(value, state)
+        if isinstance(value, datetime.datetime):
+            value = value.time()
+        return value
 
 class SOTimeCol(SOCol):
     timeFormat = '%H:%M:%S'
 
     def __init__(self, **kw):
-        timeFormat = popKey(kw, 'timeFormat')
+        timeFormat = kw.pop('timeFormat', None)
         if timeFormat:
             self.timeFormat = timeFormat
         super(SOTimeCol, self).__init__(**kw)
@@ -1316,7 +1298,7 @@ class DecimalValidator(validators.Validator):
             return None
         if isinstance(value, float):
             value = str(value)
-        if isinstance(value, (str, unicode)):
+        if isinstance(value, basestring):
             connection = state.soObject._connection
             if hasattr(connection, "decimalSeparator"):
                 value = value.replace(connection.decimalSeparator, ".")
@@ -1333,10 +1315,10 @@ class DecimalValidator(validators.Validator):
 class SODecimalCol(SOCol):
 
     def __init__(self, **kw):
-        self.size = popKey(kw, 'size', NoDefault)
+        self.size = kw.pop('size', NoDefault)
         assert self.size is not NoDefault, \
                "You must give a size argument"
-        self.precision = popKey(kw, 'precision', NoDefault)
+        self.precision = kw.pop('precision', NoDefault)
         assert self.precision is not NoDefault, \
                "You must give a precision argument"
         super(SODecimalCol, self).__init__(**kw)
@@ -1380,11 +1362,11 @@ class BinaryValidator(validators.Validator):
             if connection.dbName == "sqlite":
                 value = connection.module.decode(value)
             return value
-        if isinstance(value, (buffer_type, state.soObject._connection._binaryType)):
+        if isinstance(value, (buffer, state.soObject._connection._binaryType)):
             cachedValue = self._cachedValue
             if cachedValue and cachedValue[1] == value:
                 return cachedValue[0]
-            if isinstance(value, array_type): # MySQL
+            if isinstance(value, array): # MySQL
                 return value.tostring()
             return str(value) # buffer => string
         raise validators.Invalid("expected a string in the BLOBCol '%s', got %s %r instead" % \
@@ -1453,12 +1435,12 @@ class PickleValidator(BinaryValidator):
     def from_python(self, value, state):
         if value is None:
             return None
-        return pickle.dumps(value)
+        return pickle.dumps(value, self.pickleProtocol)
 
 class SOPickleCol(SOBLOBCol):
 
     def __init__(self, **kw):
-        self.pickleProtocol = popKey(kw, 'pickleProtocol', 1)
+        self.pickleProtocol = kw.pop('pickleProtocol', pickle.HIGHEST_PROTOCOL)
         super(SOPickleCol, self).__init__(**kw)
 
     def createValidators(self):
@@ -1466,24 +1448,25 @@ class SOPickleCol(SOBLOBCol):
             name=self.name, pickleProtocol=self.pickleProtocol)] + \
             super(SOPickleCol, self).createValidators()
 
+    def _mysqlType(self):
+        length = self.length
+        if length >= 2**24:
+            return "LONGBLOB"
+        if length >= 2**16:
+            return "MEDIUMBLOB"
+        return "BLOB"
+
 class PickleCol(BLOBCol):
     baseClass = SOPickleCol
 
 
-def popKey(kw, name, default=None):
-    if not kw.has_key(name):
-        return default
-    value = kw[name]
-    del kw[name]
-    return value
-
 def pushKey(kw, name, value):
     if not kw.has_key(name):
         kw[name] = value
 
 all = []
 for key, value in globals().items():
-    if isinstance(value, type) and (issubclass(value, Col) or issubclass(value, SOCol)):
+    if isinstance(value, type) and (issubclass(value, (Col, SOCol))):
         all.append(key)
 __all__.extend(all)
 del all
