@@ -41,8 +41,8 @@ class MSSQLConnection(DBAPI):
                 conn_str += "User Id=%s;Password=%s"
                 self.make_conn_str = lambda keys: [conn_str % (keys.host, keys.db, keys.user, keys.password)]
 
-            col.popKey(kw, "sspi")
-            col.popKey(kw, "ncli")
+            kw.pop("sspi", None)
+            kw.pop("ncli", None)
 
         else: # pymssql
             self.dbconnection = sqlmodule.connect
@@ -60,7 +60,6 @@ class MSSQLConnection(DBAPI):
         self.limit_re = re.compile('^\s*(select )(.*)', re.IGNORECASE)
         self.password = password
         self.module = sqlmodule
-        self._can_use_max_types = None
         DBAPI.__init__(self, **kw)
 
     def connectionFromURI(cls, uri):
@@ -144,21 +143,22 @@ class MSSQLConnection(DBAPI):
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
 
-    def _queryAddLimitOffset(self, query, start, end):
+    def _queryAddLimitOffset(cls, query, start, end):
         if end and not start:
             limit_str = "SELECT TOP %i" % end
 
-            match = self.limit_re.match(query)
+            match = cls.limit_re.match(query)
             if match and len(match.groups()) == 2:
                 return ' '.join([limit_str, match.group(2)])
         else:
             return query
+    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createReferenceConstraint(self, soClass, col):
         return col.mssqlCreateReferenceConstraint()
 
     def createColumn(self, soClass, col):
-        return col.mssqlCreateSQL(self)
+        return col.mssqlCreateSQL()
 
     def createIDColumn(self, soClass):
         key_type = {int: "INT", str: "TEXT"}[soClass.sqlmeta.idType]
@@ -180,7 +180,7 @@ class MSSQLConnection(DBAPI):
     def addColumn(self, tableName, column):
         self.query('ALTER TABLE %s ADD %s' %
                    (tableName,
-                    column.mssqlCreateSQL(self)))
+                    column.mssqlCreateSQL()))
 
     def delColumn(self, sqlmeta, column):
         self.query('ALTER TABLE %s DROP COLUMN %s' % (tableName.table, column.dbName))
@@ -216,7 +216,8 @@ class MSSQLConnection(DBAPI):
                                 % tableName)
         results = []
         for field, size, t, precision, scale, nullAllowed, default, defaultText, is_identity in colData:
-            if field == soClass.sqlmeta.idName:
+            # Seems strange to skip the pk column?  What if it's not 'id'?
+            if field == 'id':
                 continue
             # precision is needed for decimal columns
             colClass, kw = self.guessClass(t, size, precision, scale)
@@ -275,21 +276,3 @@ class MSSQLConnection(DBAPI):
                                    'precision': scale}
         else:
             return col.Col, {}
-
-    def server_version(self):
-        try:
-            server_version = self.queryAll("SELECT SERVERPROPERTY('productversion')")[0][0]
-            server_version = server_version.split('.')[0]
-            server_version = int(server_version)
-        except:
-            server_version = None # unknown
-        self.server_version = server_version # cache it forever
-        return server_version
-
-    def can_use_max_types(self):
-        if self._can_use_max_types is not None:
-            return self._can_use_max_types
-        server_version = self.server_version()
-        self._can_use_max_types = can_use_max_types = \
-            (server_version is not None) and (server_version >= 9)
-        return can_use_max_types
