@@ -1,19 +1,9 @@
-from __future__ import generators # for enumerate
 from sqlobject import *
 from sqlobject.sqlbuilder import func
 from sqlobject.main import SQLObjectIntegrityError
 from dbtest import *
 from dbtest import setSQLiteConnectionFactory
 
-try:
-    enumerate
-except NameError:
-    def enumerate(iterable):
-        i = 0
-        for obj in iterable:
-            yield i, obj
-            i += 1
-
 class IterTest(SQLObject):
     name = StringCol(dbName='name_col')
 
@@ -133,14 +123,6 @@ class TestSelect:
         assert func([c.n1 for c in counters]) == value
 
     def test_1(self):
-        try:
-            sum
-        except NameError:
-            def sum(sequence):
-                s = 0
-                for item in sequence:
-                    s = s + item
-                return s
         self.accumulateEqual(sum,Counter2.select(orderBy='n1'),
                              sum(range(10)) * 10)
 
